#!/bin/bash

_self_bin_name="$0"
function where_is_him () {
    SOURCE="$1"
    while [ -h "$SOURCE" ]; do # resolve $SOURCE until the file is no longer a symlink
        DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
        SOURCE="$(readlink "$SOURCE")"
        [[ $SOURCE != /* ]] && SOURCE="$DIR/$SOURCE" # if $SOURCE was a relative symlink, we need to resolve it relative to the path where the symlink file was located
    done
    DIR="$( cd -P "$( dirname "$SOURCE" )" && pwd )"
    echo -n "$DIR"
}

function where_am_i () {
    _my_path=`type -p ${_self_bin_name}`
    [[ "$_my_path" = "" ]] && where_is_him "$_self_bin_name" || where_is_him "$_my_path"
}

! source "$(where_am_i)/antidote.config.sh" && echo "Failed to source config file" && exit 2

##############################################################################3
function upload_to_smb_share () {
    # returns smb_url from stdout
    release_ver="$1"
    local_wf_dir="$2"

    # The workflow builder should copy all xaml file to there. 
    find "$local_wf_dir" -name '*.pdb' -exec rm -f '{}' ';'
    cmd="deltree /antidote-cis/$smb_namespace/$release_ver;mkdir /antidote-cis;mkdir /antidote-cis/$smb_namespace;mkdir /antidote-cis/$smb_namespace/$release_ver;mask \"\";recurse ON;prompt OFF;cd /antidote-cis/$smb_namespace/$release_ver;lcd $local_wf_dir;mput *"
    
    smbclient "$smb_provider" --user "$smb_username%$smb_password" -c "$cmd" 1>&2 || return 2
    smb_url="$smb_provider/antidote-cis/$smb_namespace/$release_ver"
    echo "$smb_url"
}

# CIS upload pkg
function cis_upload_pkg () {
    pkg_fullname="$1"
    release_ver="$2"
    pkg_path="$3"
    smb_url=`upload_to_smb_share "$release_ver" "$pkg_path"` || return $?
    echo "Submitting request to upload package $pkg_fullname:$release_ver, using smb:$smb_url..."
    curl --data '[{"Cloud":"Public","PackageType":"'"$pkg_fullname"'","Version":"'"$release_ver"'","PackageId":"00000000-0000-0000-0000-000000000000","ActionType":501,"Notes":"antidote-auto-upload","ActionArgs":{"PackageSourcePath":"'"$smb_url"'","ScriptParameters":""}}]' "https://beta-cps.trafficmanager.net/cis.svc/Public/$pkg_fullname/Packages/Request" --cookie "$cis_cookie" -H 'content-type: application/json' --max-time 30
    return $?
}

function try_grep_pkgid () {
    pkg_fullname="$1"
    release_ver="$2"
    resp=`curl -s "https://beta-cps.trafficmanager.net/cis.svc/Public/$pkg_fullname/Packages?null" --cookie "$cis_cookie" --max-time 30` || return $?
    [[ "$resp" = "" ]] && echo "Invalid api response. Is cookie expired or not correctly set?" && return 2
    ! echo "$resp" | json2table | grep -F " $release_ver " > /dev/null && echo "Error: release_ver $release_ver doesn't exist at all" && return 2

    echo "DEBUG: resp=$resp"
    # output pkgid
    echo "$resp" | json2table | grep -v 00000000-0000-0000-0000-000000000000 | grep -F " $release_ver " | cut -d '|' -f 5 | tr -d ' '
    return $? # return false if grep -F failed, which indicates that package is still uploading...
}


function cis_release_pkg () {
    pkg_fullname="$1"
    release_ver="$2"
    pkg_path="$3"
    echo "Release package $pkg_fullname:$release_ver..."
    pkg_id=`try_grep_pkgid "$pkg_fullname" "$release_ver"`
    curl --data '[{"Cloud":"Public","PackageType":"'"$pkg_fullname"'","Version":"'"$release_ver"'","PackageId":"'"$pkg_id"'","ActionType":505,"Notes":"antidote-auto-release","ActionArgs":{"PackageSourcePath":null,"ScriptParameters":""}}]' "https://beta-cps.trafficmanager.net/cis.svc/Public/$pkg_fullname/Packages/Request" --cookie "$cis_cookie" -H 'content-type: application/json' --max-time 30
    return $?
}

# CIS show pkg list
# GET https://beta-cps.trafficmanager.net/cis.svc/Public/M365FleetAGCTest.E2EPOCBuildPortalWorkflow/Packages?null

# CIS wait upload pkg success
function cis_wait_for_upload () {
    pkg_fullname="$1"
    release_ver="$2"
    echo "Waiting for CIS to create package..."
    while true; do
        try_grep_pkgid "$pkg_fullname" "$release_ver" && break # success
        echo -n .
        sleep 5
    done
}


# receive arguments
release_ver="$1"
local_wf_dir=~/tmp/testshit/net472
pkg_fullname="$cis_namespace.$cis_default_pkgname"


cis_upload_pkg "$pkg_fullname" "$release_ver" "$local_wf_dir" &&
cis_wait_for_upload "$pkg_fullname" "$release_ver" &&
cis_release_pkg "$pkg_fullname" "$release_ver"




